using System;
using System.Collections;

namespace VaultCmdLineClient
{
	public class Help
	{
		XMLOutputWriter _xml = null;
		Hashtable commandHelpHash = new Hashtable();
		Hashtable optionHelpHash = new Hashtable();

		public Help(XMLOutputWriter newxml)
		{
			_xml = newxml;

			// Help for commands

			commandHelpHash.Add(Command.ADD, new CommandHelpEntry(
				"repositoryfolder localpath ...",
				"Add files or folders to the repository",
				"ADD will add the files or folders specified by localpath to the repository\nat the specified repositoryfolder.",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.ADDREPOSITORY, new CommandHelpEntry(
				"repositoryname",
				"Add a new repository to the server",
				"ADDREPOSITORY will create a new repository on the server named\nrepositoryname",
				new Option[] { }
				));

			commandHelpHash.Add(Command.ADDUSER, new CommandHelpEntry(
				"username password emailaddress",
				"Add a new user to the server",
				"ADDUSER will create a new user on the server, specified by username,\npassword and emailaddress.",
				new Option[] { }
				));

			// Help for options
			commandHelpHash.Add(Command.BATCH, new CommandHelpEntry(
				"- | filename",
				"Place the command line client in batch mode", 
				"Batching allows multiple vault commands to be executed within one 'run'.\n\nUse - (hyphen) to specify the batch commands can be found on standard input.\nOtherwise, provide the name of a file where the vault commands can be located.\nIn batch mode, please provide only one vault command per line.", 
				new Option[] { } 
				));

			commandHelpHash.Add(Command.BLAME, new CommandHelpEntry(
				"repositorypath linenumber",
				"Determine the user that modified a given line",
				"BLAME will determine the last user that modified\nthe given line of a file.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.BRANCH, new CommandHelpEntry(
				"repositoryfolder torepositoryfolder",
				"Create a branch for a repository folder",
				"BRANCH will create a branch for the repository folder specified\nby repositoryfolder at torepositoryfolder.",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.CHECKOUT, new CommandHelpEntry(
				"repositoryfile ...",
				"Checkout files from the repository",
				"CHECKOUT will checkout files from the repository.",
				new Option[] { Option.EXCLUSIVE, Option.MERGE, Option.SETFILETIME }
				));

			commandHelpHash.Add(Command.CLOAK, new CommandHelpEntry(
				"repositoryfolder ...",
				"Cloak repository folders",
				"CLOAK will cloak a repository folder specified by repositoryfolder,\nmeaning that folder will be ignored during recursive GET\noperations.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.COMMIT, new CommandHelpEntry(
				"[repositorypath ...]",
				"Commit items in the pending changeset",
				"COMMIT will commit the items in the pending changeset list specified\nby repositorypath(s).  If no items are specified on the command line,\nall items in the pending changeset list are committed.",
				new Option[] { Option.COMMENT, Option.KEEPCHECKEDOUT, Option.UNCHANGED }
				));

			commandHelpHash.Add(Command.CREATEFOLDER, new CommandHelpEntry(
				"repositorypath",
				"Create a new folder in the repository",
				"CREATEFOLDER will create a folder in the repository at the point\nspecified by repositorypath.",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.DELETE, new CommandHelpEntry(
				"repositorypath ...",
				"Delete files or folders from the repository",
				"DELETE will delete the files or folders specified by the\nrepositorypath(s) from the repository.",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.DIFF, new CommandHelpEntry(
				"OBJECT_1 OBJECT2", 
				"Perform a diff against the two objects.", 
				"{0} will run the two objects through a 'diff' program.", 
				new Option[] { Option.COMPARETO, Option.NORECURSIVE, Option.VAULTDIFF, Option.VAULTDIFF_OPTIONS }
				));

			commandHelpHash.Add(Command.FORGETLOGIN, new CommandHelpEntry(
				"",
				"Remove locally stored login information",
				"FORGETLOGIN will remove all login information being locally stored\nby the REMEMBERLOGIN command.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.GET, new CommandHelpEntry(
				"repositorypath ...",
				"Retrieve files or folders from the repository",
				"GET will retrieve the latest version of files or folders in the\nrepository as specified by repositorypath(s).  The files will be stored\nin their corresponding working folders on the local system.",
				new Option[] { Option.DESTPATH, Option.MAKEWRITABLE, Option.MAKEREADONLY, Option.MERGE, Option.NOCLOAKS, Option.NORECURSIVE, Option.SETFILETIME }
				));

			commandHelpHash.Add(Command.GETLABEL, new CommandHelpEntry(
				"repositorypath label [labelpath]",
				"Retrieve files or folders specified by label",
				"GETLABEL will retrieve files or folders specified by repositorypath\nat the version at which they were labelled with the specified label.\nUse the -destpath flag to specify folder to get files to.\nDo not set labelpath unless prompted to.",
				new Option[] { Option.DESTPATH, Option.MAKEWRITABLE, Option.MAKEREADONLY, Option.MERGE, Option.NORECURSIVE, Option.SETFILETIME }
				));

			commandHelpHash.Add(Command.GETLABELDIFFS, new CommandHelpEntry(
				"repositorypath label1 [labelpath2]",
				"Retrieve history for files changed between specified labels.",
				"GETLABELDIFFS will retrieve the histories for all files in the specified repository folder between the specified labels.\n" + 
				"If only one label is specified, retrieves history from that label to present.",
				new Option[] { Option.ROWLIMIT, Option.VERBOSE}
				));

			commandHelpHash.Add(Command.GETVERSION, new CommandHelpEntry(
				"version repositorypath localfolder",
				"Retrieve files or folder at a specified version",
				"GETVERSION will retrieve a file or folder specified by repositorypath\nat the given version and place it into localfolder.",
				new Option[] { Option.MAKEWRITABLE, Option.MAKEREADONLY, Option.MERGE, Option.NOCLOAKS, Option.NORECURSIVE, Option.SETFILETIME }
				));

			commandHelpHash.Add(Command.GETWILDCARD, new CommandHelpEntry(
				"repositoryfolder wildcard ...",
				"Retrieve files whose names match a wildcard",
				"GETWILDCARD will retrieve all files within the folder specified by\nrepositoryfolder whose name matches one of the wildcards specified.\nYou may use '?' to match a single character or '*' to match a range of\ncharacters.",
				new Option[] { Option.DESTPATH, Option.MAKEWRITABLE, Option.MAKEREADONLY, Option.MERGE, Option.NOCLOAKS, Option.NORECURSIVE, Option.SETFILETIME }
				));

			commandHelpHash.Add(Command.HELP, new CommandHelpEntry(
				"[command]",
				"Displays this page or help for a specific command",
				"HELP will display a list of all available commands, or detailed help for\na specified command.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.HISTORY, new CommandHelpEntry(
				"repositorypath",
				"Shows history for an item in the repository",
				"HISTORY will display all committed actions to a file or folder in the\nrepository specified by repositorypath.",
				new Option[] { Option.ROWLIMIT }
				));

			commandHelpHash.Add(Command.LABEL, new CommandHelpEntry(
				"repositorypath labelname",
				"Labels current version of repositorypath with labelname",
				"Assigns a label to the current version of repositorypath which can\nbe used later for GETLABEL requests.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.LISTALLBRANCHPOINTS, new CommandHelpEntry(
				"",
				"Show all branch points in the repository",
				"LISTALLBRANCHPOINTS will display all branch points in the repository.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.LISTCHANGESET, new CommandHelpEntry(
				"",
				"Show the contents of the pending changeset",
				"LISTCHANGESET will display the contents of the pending changeset.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.LISTCHECKOUTS, new CommandHelpEntry(
				"",
				"Show all check out items for the current repository",
				"LISTCHECKOUTS will display the check out items for the current repository.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.LISTFOLDER, new CommandHelpEntry(
				"repositoryfolder",
				"Show contents and status of a repository folder",
				"LISTFOLDER will display the contents of the folder specified by\nrepositoryfolder, including the status of any working folders",
				new Option[] { Option.NORECURSIVE }
				));

			commandHelpHash.Add(Command.LISTREPOSITORIES, new CommandHelpEntry(
				"",
				"List available repositories on the server",
				"LISTREPOSITORIES will list all repositories on the server.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.LISTUSERS, new CommandHelpEntry(
				"",
				"List all users on the server",
				"LISTUSERS will list all users on the server.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.LISTWORKINGFOLDERS, new CommandHelpEntry(
				"",
				"List local working folder associations",
				"LISTWORKINGFOLDERS will list all current associations between repository\nfolders and local working folders.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.MOVE, new CommandHelpEntry(
				"repositorypath destrepositoryfolder",
				"Move a file or folder to a different folder",
				"MOVE will move the file or folder specified by repositorypath into the\nrepository folder specified by destrepositoryfolder.",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.PIN, new CommandHelpEntry(
				"repositorypath version",
				"Pin a file or folder",
				"PIN will pin the file or folder specified by repositorypath at the\nspecified version.",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.REMEMBERLOGIN, new CommandHelpEntry(
				"",
				"Store authentication information",
				"REMEMBERLOGIN will store server, repository and authentication information\non the local host so that it does not need to be reentered.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.RENAME, new CommandHelpEntry(
				"repositorypath destrepositorypath",
				"Rename a file or folder",
				"RENAME will rename a file or folder specified by repositorypath to the new\nname destrepositorypath.",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.SETLABELWORKINGFOLDER, new CommandHelpEntry(
				"repositoryfolder label localfolder [labelpath]",
				"Set a working folder association for a label",
				"SETLABELWORKINGFOLDER will set the working folder for the specified repository\nfolder at a label to be the local folder.  All future operations on\nrepositoryfolder will use localfolder as a working folder by default.\nDo not set labelpath unless prompted to.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.SETWORKINGFOLDER, new CommandHelpEntry(
				"repositoryfolder localfolder",
				"Set a working folder association",
				"SETWORKINGFOLDER will set the working folder for the specified repository\nfolder to be the local folder.  All future operations on repositoryfolder\nwill use localfolder as a working folder by default.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.SHARE, new CommandHelpEntry(
				"repositorypath destrepositoryfolder",
				"Share a file or folder into a different folder",
				"SHARE will share a file or folder specified by repositorypath into the\nrepository folder specified by destrepositoryfolder",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.UNCLOAK, new CommandHelpEntry(
				"repositoryfolder",
				"Uncloak a folder",
				"UNCLOAK will uncloak specified by repositoryfolder.",
				new Option[] { }
				));

			commandHelpHash.Add(Command.UNDOCHANGESETITEM, new CommandHelpEntry(
				"changeset item id", 
				"Remove an item from the change set", 
				"UNDOCHANGESETITEM will remove the change set item from the change set list.",
				new Option[] {}
				));

			commandHelpHash.Add(Command.UNDOCHECKOUT, new CommandHelpEntry(
				"repositorypath",
				"Undo checkout (revert changes)",
				"UNDOCHECKOUT will undo a checkout, reverting changes back to the data in\nthe repository.",
				new Option[] { Option.NORECURSIVE }
				));

			commandHelpHash.Add(Command.UNPIN, new CommandHelpEntry(
				"repositorypath",
				"Unpin a file or folder",
				"UNPIN will unpin the file or folder specified by repositorypath.",
				new Option[] { Option.COMMENT, Option.COMMIT }
				));

			commandHelpHash.Add(Command.UNSETWORKINGFOLDER, new CommandHelpEntry(
				"repositoryfolder",
				"Remove a working folder association",
				"UNSETWORKINGFOLDER will remove the local working folder association for\nthe specified repositoryfolder.",
				new Option[] { }
				));


			///////////////////////////////
			//  help for options
			///////////////////////////////

			optionHelpHash.Add(Option.BEGINDATE, new OptionHelpEntry(
				"yyyy-mm-dd",
				"Date to begin history at"
				));

			optionHelpHash.Add(Option.COMMENT, new OptionHelpEntry(
				"commentstring",
				"Checkin comment"
				));

			optionHelpHash.Add(Option.COMMIT, new OptionHelpEntry(
				"",
				"Commit this action automatically"
				));

			optionHelpHash.Add(Option.COMPARETO, new OptionHelpEntry(
				"[current|label|lastget|local|repository]",
				"Type of comparison should be made against your local working version.\n\ncurrent\t- the current repository item\nlabel\t- the version found within the given label\nlastget\t- the last version retrieved from the repository\nlocal\t- any local file system item\nrepository- any item within the repository"
				));

			optionHelpHash.Add(Option.DESTFOLDER, new OptionHelpEntry(
				"localfolder",
				"Use localfolder for actions instead of any existing working folder"
				));

			optionHelpHash.Add(Option.DESTPATH, new OptionHelpEntry(
				"localfolder",
				"Use localfolder for actions instead of any existing working folder"
				));

			optionHelpHash.Add(Option.ENDDATE, new OptionHelpEntry(
				"yyyy-mm-dd",
				"Date to end history display at"
				));

			optionHelpHash.Add(Option.EXCLUSIVE, new OptionHelpEntry(
				"",
				"Will check out items exclusively"
				));

			optionHelpHash.Add(Option.HOST, new OptionHelpEntry(
				"host",
				"Hostname of the vault server to connect to - also see " + Option.SERVER
				));

			optionHelpHash.Add(Option.KEEPCHECKEDOUT, new OptionHelpEntry(
				"keepcheckedout",
				"all files remain checked out upon commit"
				));

			optionHelpHash.Add(Option.LEAVEFILE, new OptionHelpEntry(
				"leavefile",
				"will leave the file unchanged on an undo checkout"
				));

			optionHelpHash.Add(Option.MAKEWRITABLE, new OptionHelpEntry(
				"",
				"Make all files writable after retrieval"
				));

			optionHelpHash.Add(Option.MAKEREADONLY, new OptionHelpEntry(
				"",
				"Make all files read-only after retrieval"
				));

			optionHelpHash.Add(Option.MERGE, new OptionHelpEntry(
				"[automatic|later|overwrite]",
				string.Format("The action to take when updating a local file with new content.\n\nautomatic*\t- attempt to merge changes from the server\nlater\t\t- do not overwrite an existing, modified file\noverwrite\t\t- overwrite the local file with the server's file\n\n* - only applies to {0} and {1} commands\n", Command.GET, Command.GETWILDCARD)
				));

			optionHelpHash.Add(Option.NOCLOAKS, new OptionHelpEntry(
				"",
				"Performs actions on all folders even if they were previously cloaked"
				));

			optionHelpHash.Add(Option.NORECURSIVE, new OptionHelpEntry(
				"",
				"Do not act recursively on folders"
				));

			optionHelpHash.Add(Option.NOSSL, new OptionHelpEntry(
				"",
				"Disable SSL for server connection"
				));

			optionHelpHash.Add(Option.OUT, new OptionHelpEntry(
				"filename",
				"Will write output log in XML format to filename"
				));

			optionHelpHash.Add(Option.PASSWORD, new OptionHelpEntry(
				"password",
				"Password to use when connecting to server"
				));

			optionHelpHash.Add(Option.REPOSITORY, new OptionHelpEntry(
				"repositoryname",
				"Repository to connect to"
				));

			optionHelpHash.Add(Option.REQUIRECHECKOUT, new OptionHelpEntry(
				"",
				"Requires checkouts of files before commit can be performed on\nmodified files"
				));

			optionHelpHash.Add(Option.REVERTFILE, new OptionHelpEntry(
				"revertfile",
				"will revert the file to its last known version for an undo checkout"
				));

			optionHelpHash.Add(Option.ROWLIMIT, new OptionHelpEntry(
				"limitnumber",
				"Limits the number of rows returned for a history query to\nlimitnumber"
				));

			optionHelpHash.Add(Option.SERVER, new OptionHelpEntry(
				"server",
				"name of the vault server to connect to - also see " + Option.HOST
				));

			optionHelpHash.Add(Option.SETFILETIME, new OptionHelpEntry(
				"checkin|current|modification",
				"Sets the time of the local file.\n\ncheckin\t\t- use the last checkin time\ncurrent\t\t- use the current system time\nmodification\t- use the file's last modified time"
				));

			optionHelpHash.Add(Option.SSL, new OptionHelpEntry(
				"",
				"Enables SSL for server connection"
				));

			optionHelpHash.Add(Option.UNCHANGED, new OptionHelpEntry(
				"checkin|leavecheckedout|undocheckout",
				"Action to perform on an unchanged, checked-out file during commit.\nCheck the file in unmodified, leave the file checked out, or\nundo the checkout of the file"
				));

			optionHelpHash.Add(Option.URL, new OptionHelpEntry(
				"serverurl",
				"URL of the server"
				));

			optionHelpHash.Add(Option.USER, new OptionHelpEntry(
				"username",
				"Username to use when connecting to server - also see " + Option.USERNAME
				));

			optionHelpHash.Add(Option.USERNAME, new OptionHelpEntry(
				"username",
				"Username to use when connecting to server - also see " + Option.USER
				));

			optionHelpHash.Add(Option.VAULTDIFF, new OptionHelpEntry(
				"[absolute path to comparison utility]",
				"Use this comparison utility for differences."
				));

			optionHelpHash.Add(Option.VAULTDIFF_OPTIONS, new OptionHelpEntry(
				"\"[any options]\"", 
				"Options for the comparison utility.\n\tUse double quotes for multiple options."
				));

			optionHelpHash.Add(Option.VERBOSE, new OptionHelpEntry(
				"",
				"Turn on verbose mode"
				));
		}

		public void Copyright(XMLOutputWriter xml)
		{
			xml.WriteContent("SourceGear Vault Command Line Client");
			xml.WriteContent("Copyright (c) 2003 SourceGear Corporation.  All Rights Reserved.");
			xml.WriteContent();
		}

		public void Write()
		{
			_xml.Begin("usage");

			Copyright(_xml);

			_xml.WriteContent("usage: gvault commandname [options] [parameters]");
			_xml.WriteContent();

			_xml.WriteContent("This is a list of possible commands:");

			foreach (Command cmd in Enum.GetValues(typeof(Command)))
			{
				if(cmd == Command.NONE || cmd == Command.INVALID)
					continue;

				string cmdName = cmd.ToString().ToUpper();

				CommandHelpEntry he = (CommandHelpEntry) commandHelpHash[cmd];

				if(he != null)
				{
					_xml.WriteContent(string.Format("  {0,-22}{1}", cmdName, he.ShortDetails));
				}
				else
				{
					_xml.WriteContent(string.Format("  {0,-22}", cmdName));
				}
			}

			_xml.WriteContent();
			_xml.WriteContent("For help with a specific command, run 'vault.exe HELP commandname'.");

			_xml.End();
		}

		public void Write(string cmdName)
		{
			Command cmd = Command.INVALID;
			foreach(Command testcmd in Enum.GetValues(typeof(Command)))
			{
				if(testcmd.ToString().ToLower() == cmdName.ToLower())
					cmd = testcmd;
			}

			if(cmd == Command.INVALID || cmd == Command.NONE)
			{
				throw new UsageException(string.Format("unknown command: {0} - run 'vault.exe HELP' for help", cmdName.ToUpper()));
			}

			CommandHelpEntry he = (CommandHelpEntry) commandHelpHash[cmd];

			_xml.Begin("usage");

			Copyright(_xml);

			if(he.OptionList.Length == 0)
			{
				_xml.WriteContent(string.Format("usage: vault.exe {0} {1}", cmdName.ToUpper(), he.Usage));
			}
			else
			{
				_xml.WriteContent(string.Format("usage: vault.exe {0} [options] {1}", cmdName.ToUpper(), he.Usage));
			}
			_xml.WriteContent();

			_xml.WriteContent(he.LongDetails);
			_xml.WriteContent();

			if(VaultCmdLineClient.CommandNeedsAdmin(cmd))
			{
				_xml.WriteContent("This command requires administrative privileges.");
				_xml.WriteContent();
			}

			if(VaultCmdLineClient.CommandNeedsLogin(cmd))
			{
				Option[] optionList = null;

				if(VaultCmdLineClient.CommandNeedsRepositorySpecified(cmd))
				{
					optionList = new Option[] { Option.HOST, Option.SSL, Option.USER, Option.PASSWORD, Option.REPOSITORY };
				}
				else
				{
					optionList = new Option[] { Option.HOST, Option.SSL, Option.USER, Option.PASSWORD };
				}

				_xml.WriteContent("Server and authentication information is specified by:");

				foreach(Option opt in optionList )
				{
					string optName = opt.ToString().ToLower();

					OptionHelpEntry ohe = (OptionHelpEntry) optionHelpHash[opt];

					if(ohe != null)
					{
						string details = ohe.Details.Replace("\n", "\n      ");
						_xml.WriteContent(string.Format("  -{0} {1}\n      {2}", optName, ohe.Usage, details));
					}
					else
					{
						_xml.WriteContent(string.Format("  -{0}", optName));
					}
				}

				_xml.WriteContent();
			}

			if(he.OptionList.Length > 0)
			{
				_xml.WriteContent("This is a list of possible options:");

				foreach(Option opt in he.OptionList)
				{
					string optName = opt.ToString().ToLower();

					OptionHelpEntry ohe = (OptionHelpEntry) optionHelpHash[opt];

					if(ohe != null)
					{
						string details = ohe.Details.Replace("\n", "\n      ");
						_xml.WriteContent(string.Format("  -{0} {1}\n      {2}", optName, ohe.Usage, details));
					}
					else
					{
						_xml.WriteContent(string.Format("  -{0}", optName));
					}
				}
			}

			_xml.End();
		}
	}
}
